//+------------------------------------------------------------------+
//|                                        patternmatcher.common.mqh |
//|                                   Copyright  2010, Bernd Kreuss |
//|                                             Version: 2010.1.30.1 |
//+------------------------------------------------------------------+
#property copyright "Bernd Kreuss"
#property link      "mailto:7ibt@arcor.de"

/**
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include <common_functions.mqh>

int lines_plotted;
double last_plotted_y_0;


/**
* The pattern function and all the CPU intensive pattern matching 
* stuff is implemented in Pascal to gain a few orders of magnitude
* performance increasement. See the dll source code for the 
* documentation of the functions listed below. 
*/
#import "pattern.dll"
   void setRatesArray(double& rates[][6], int num_bars);
   void setParams(double p1, double p2, double p3);
   double pattern(double x);
   double calcPriceRange(int width, int last_bar);
   void scalePatternY(double& scale[2], int width, int last_bar);
   int findBest(int range);
   double lastError();
#import


/**
* tell the dll the address of a memory location
* containing the current currency rates of the active chart
*/
void setRates(){
   static double history[][6];
   int num_bars = ArrayCopyRates(history, NULL, 0);
   setRatesArray(history, num_bars);
}

/**
* Plot the pattern into the chart
* width determines the size of the pattern.
*/
double plot(int width, bool permanent=false, int last_bar=0){
   int i;
   double p, p1;
   int sign = 1;
   string name;
   double scale[2]; // 0:y_0 1:y_scale
   scalePatternY(scale, width, last_bar);
   last_plotted_y_0 = scale[0]; // for later use when plotting labels
      
   if (width < 0){
      width = -width;
      sign = -1;
   }
   
   for (i=0; i<=width; i++){
      p1 = p;
      p = scale[0] + sign * scale[1] * pattern(100.0 * i / width);
      if (i>0){
         if (!permanent){
            name = "pattern" + (lines_plotted + i);
         }else{
            name = "_pattern" + Time[last_bar] + i;
         }
         line(name, Time[last_bar + i - 1], p1, Time[last_bar + i], p); 
         ObjectSet(name, OBJPROP_WIDTH, 2);
         ObjectSet(name, OBJPROP_BACK, True);
         if (permanent){
            if (sign>0){
               ObjectSet(name, OBJPROP_COLOR, Lime);
            }else{
               ObjectSet(name, OBJPROP_COLOR, Red);
            }
         }else{
            ObjectSet(name, OBJPROP_COLOR, Yellow);
         }
      }
   }
   lines_plotted += width + 1;
}

/**
* remove the pattern plot(only the non-permanent ones)
*/
void clearPlot(){
   int i;
   for (i=0; i<=lines_plotted; i++){
      ObjectDelete("pattern"+i);
   }
   lines_plotted = 0;
}

